#if defined(_WIN32)

/* Windows */
# include <windows.h>
# include <tchar.h>
# define AVR2_VIA_USB_SUPPORTED
# define AVR2_VIA_ADXDMA_SUPPORTED

#else

/* Linux or VxWorks */
# define _T(x) x
# define _tscanf_s scanf
# define _tcschr(s, c) strchr(s, c)
# define _tcslen(s) strlen(s)
# define _tprintf printf
# define _stscanf_s sscanf
# define FALSE (0)
# define TRUE (1)

# if defined(__VXWORKS__) || defined(__vxworks)
#   define ADB3_LIBRARY_STATIC
#   define AVR2_VIA_USB_SUPPORTED
#   define AVR2_LIBRARY_STATIC
# else
#   define AVR2_VIA_USB_SUPPORTED
#   define AVR2_VIA_SMBUS_SUPPORTED
/* NOTE: Makefile defines AVR2_VIA_ADXDMA_SUPPORTED */
/* NOTE: Makefile may or may not define AVR2_VIA_IPMI_SUPPORTED */
#   if !defined(AVR2_LIBRARY_STATIC) || !defined(AVR2S_LIBRARY_STATIC) || !defined(AVR2IPMI_LIBRARY_STATIC) || !defined(ADB3_LIBRARY_STATIC)
/* Needed for dlopen() */
#     include <dlfcn.h>
#   endif
# endif

#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>

#include <sdk_common.h>
#include <platform_endian.h>
#include <platform_time.h>

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

#include "connection.h"
#include "exit_code.h"
#include "firmware.h"
#include "product_id.h"
#include "vpd.h"

#define UC_ADB3_REBOOT_DELAY_MS   (1000) /* Time allowed for uC to reboot itself when entering/exiting Service Mode when communicating via ADB3 Driver (PCIe) */
#define UC_ADXDMA_REBOOT_DELAY_MS (1000) /* Time allowed for uC to reboot itself when entering/exiting Service Mode when communicating via ADXDMA Driver (PCIe) */
#define UC_PSUART_REBOOT_DELAY_MS (1000) /* Time allowed for uC to reboot itself when entering/exiting Service Mode when communicating via PS UART */
#define UC_USBCOM_REBOOT_DELAY_MS (5000) /* Time allowed for uC to reboot itself when entering/exiting Service Mode when communicating via USB */
#define UC_SMBUS_REBOOT_DELAY_MS  (1000) /* Time allowed for uC to reboot itself when entering/exiting Service Mode when communicating via SMBUS */
#define UC_IPMI_REBOOT_DELAY_MS   (1000) /* Time allowed for uC to reboot itself when entering/exiting Service Mode when communicating via IPMI+SMBUS*/

#if defined(AVR2_VIA_USB_SUPPORTED)

/* Signatures for AVR2 API functions, when dynamically loading the DLL/shared object */
typedef AVR2_STATUS AVR2_CALLING_CONVENTION FN_AVR2_Close(
  AVR2_HANDLE             hDevice);

typedef AVR2_STATUS AVR2_CALLING_CONVENTION FN_AVR2_Command(
  AVR2_HANDLE             hDevice,
  _AVR2_UINT32            flags,
  _AVR2_UINT32            timeoutUs,
  _AVR2_UINT32            commandLength,
  const _AVR2_UINT8*      pCommand,
  _AVR2_UINT32            responseLength,
  _AVR2_UINT8*            pResponse,
  _AVR2_UINT32*           pActualResponseLength);

typedef const TCHAR* AVR2_CALLING_CONVENTION FN_AVR2_GetStatusString(
  AVR2_STATUS             code,
  _AVR2_BOOL              bShort);

typedef AVR2_STATUS AVR2_CALLING_CONVENTION FN_AVR2_OpenPort(
  const TCHAR*            pPortName,
  AVR2_HANDLE*            phDevice);

typedef AVR2_STATUS AVR2_CALLING_CONVENTION FN_AVR2_SetPortBaud(
  AVR2_HANDLE             hDevice,
  _AVR2_UINT32            nBaudRate);

/* Structure to contain pointers to AVR2 API functions */
typedef struct _API_AVR2 {
  FN_AVR2_Close*           pfnAVR2_Close;
  FN_AVR2_Command*         pfnAVR2_Command;
  FN_AVR2_GetStatusString* pfnAVR2_GetStatusString;
  FN_AVR2_OpenPort*        pfnAVR2_OpenPort;
  FN_AVR2_SetPortBaud*     pfnAVR2_SetPortBaud;
} API_AVR2;

#endif


#if defined(AVR2_VIA_SMBUS_SUPPORTED)

/* Signatures for AVR2S API functions, when dynamically loading the DLL/shared object */
typedef AVR2S_STATUS AVR2S_CALLING_CONVENTION FN_AVR2S_Close(
  AVR2S_HANDLE             hDevice);

typedef AVR2S_STATUS AVR2S_CALLING_CONVENTION FN_AVR2S_Command(
  AVR2S_HANDLE             hDevice,
  _AVR2S_UINT32            flags,
  _AVR2S_UINT32            timeoutUs,
  _AVR2S_UINT32            commandLength,
  const _AVR2S_UINT8*      pCommand,
  _AVR2S_UINT8             chipAddrOverride,
  _AVR2S_UINT32            responseLength,
  _AVR2S_UINT8*            pResponse,
  _AVR2S_UINT32*           pActualResponseLength);

typedef const TCHAR* AVR2S_CALLING_CONVENTION FN_AVR2S_GetStatusString(
  AVR2S_STATUS             code,
  _AVR2S_BOOL              bShort);

typedef AVR2S_STATUS AVR2S_CALLING_CONVENTION FN_AVR2S_OpenDevice(
  const TCHAR*             pDeviceName,
  _AVR2S_UINT8             chipAddr,
  AVR2S_HANDLE*            phDevice);

/* Structure to contain pointers to AVR2S API functions */
typedef struct _API_AVR2S {
  FN_AVR2S_Close*           pfnAVR2S_Close;
  FN_AVR2S_Command*         pfnAVR2S_Command;
  FN_AVR2S_GetStatusString* pfnAVR2S_GetStatusString;
  FN_AVR2S_OpenDevice*      pfnAVR2S_OpenDevice;
} API_AVR2S;

#endif


#if defined(AVR2_VIA_IPMI_SUPPORTED)

/* Signatures for AVR2IPMI API functions, when dynamically loading the DLL/shared object */
typedef AVR2IPMI_STATUS AVR2IPMI_CALLING_CONVENTION FN_AVR2IPMI_CloseSession(
  AVR2IPMI_SESSION            hSession);

typedef AVR2IPMI_STATUS AVR2IPMI_CALLING_CONVENTION FN_AVR2IPMI_Command(
  AVR2IPMI_SESSION            hSession,
  _AVR2IPMI_UINT32            flags,
  _AVR2IPMI_UINT32            timeoutUs,
  _AVR2IPMI_UINT32            commandLength,
  const _AVR2IPMI_UINT8*      pCommand,
  _AVR2IPMI_UINT8             chipAddrOverride,
  _AVR2IPMI_UINT32            responseLength,
  _AVR2IPMI_UINT8*            pResponse,
  _AVR2IPMI_UINT32*           pActualResponseLength);

typedef const TCHAR** AVR2IPMI_CALLING_CONVENTION FN_AVR2IPMI_GetBackends(
  void);

typedef void AVR2IPMI_CALLING_CONVENTION FN_AVR2IPMI_GetErrorDetail(
  AVR2IPMI_ERROR_DETAIL*      pErrorDetail);

typedef const TCHAR* AVR2IPMI_CALLING_CONVENTION FN_AVR2IPMI_GetStatusString(
  AVR2IPMI_STATUS             code,
  _AVR2IPMI_BOOL              bShort);

typedef AVR2IPMI_STATUS AVR2IPMI_CALLING_CONVENTION FN_AVR2IPMI_OpenSession(
  const AVR2IPMI_OPTIONS*     pOptions,
  AVR2IPMI_SESSION*           phSession);

typedef AVR2IPMI_STATUS AVR2IPMI_CALLING_CONVENTION FN_AVR2IPMI_ParseOptions(
  const TCHAR*                pOptionsString,
  AVR2IPMI_OPTIONS*           pOptions);

/* Structure to contain pointers to AVR2IPMI API functions */
typedef struct _API_AVR2IPMI {
  FN_AVR2IPMI_CloseSession*    pfnAVR2IPMI_CloseSession;
  FN_AVR2IPMI_Command*         pfnAVR2IPMI_Command;
  FN_AVR2IPMI_GetBackends*     pfnAVR2IPMI_GetBackends;
  FN_AVR2IPMI_GetErrorDetail*  pfnAVR2IPMI_GetErrorDetail;
  FN_AVR2IPMI_GetStatusString* pfnAVR2IPMI_GetStatusString;
  FN_AVR2IPMI_OpenSession*     pfnAVR2IPMI_OpenSession;
  FN_AVR2IPMI_ParseOptions*    pfnAVR2IPMI_ParseOptions;
} API_AVR2IPMI;

#endif


#if defined(AVR2_VIA_ADXDMA_SUPPORTED)

/* Signatures for ADXDMA API functions, when dynamically loading the DLL/shared object */

typedef ADXDMA_STATUS ADXDMA_CALLING_CONVENTION FN_ADXDMA_CloseBC(
  ADXDMA_HBC                 hBC);

typedef ADXDMA_STATUS ADXDMA_CALLING_CONVENTION FN_ADXDMA_CommandUC(
  ADXDMA_HBC                 hBC,
  _ADXDMA_UINT32             flags,
  _ADXDMA_UINT32             timeoutUs,
  _ADXDMA_UINT32             commandLength,
  const _ADXDMA_UINT8*       pCommand,
  _ADXDMA_UINT32             responseLength,
  _ADXDMA_UINT8*             pResponse,
  _ADXDMA_UINT32*            pActualResponseLength);

typedef ADXDMA_STATUS ADXDMA_CALLING_CONVENTION FN_ADXDMA_GetUCStatus(
  ADXDMA_HBC                 hBC,
  ADXDMA_UC_STATUS*          pUCStatus);

typedef const TCHAR* ADXDMA_CALLING_CONVENTION FN_ADXDMA_GetStatusString(
  ADXDMA_STATUS              code,
  _ADXDMA_BOOL               bShort);

typedef ADXDMA_STATUS ADXDMA_CALLING_CONVENTION FN_ADXDMA_OpenBC(
  ADXDMA_HDEVICE             hParentDevice,
  unsigned int               deviceIndex,
  _ADXDMA_BOOL               bPassive,
  ADXDMA_HBC*                phBC);

/* Structure to contain pointers to ADXDMA API functions */
typedef struct _API_ADXDMA {
  FN_ADXDMA_CloseBC*         pfnADXDMA_CloseBC;
  FN_ADXDMA_CommandUC*       pfnADXDMA_CommandUC;
  FN_ADXDMA_GetStatusString* pfnADXDMA_GetStatusString;
  FN_ADXDMA_GetUCStatus*     pfnADXDMA_GetUCStatus;
  FN_ADXDMA_OpenBC*          pfnADXDMA_OpenBC;
} API_ADXDMA;

#endif


/* Signatures for ADB3 API functions, when dynamically loading the DLL/shared object */

typedef ADB3_STATUS ADB3_CALLING_CONVENTION FN_ADB3_Close(
  ADB3_HANDLE             hCard);

typedef ADB3_STATUS ADB3_CALLING_CONVENTION FN_ADB3_CommandAVR2(
  ADB3_HANDLE             hCard,
  _ADB3_UINT32            flags,
  _ADB3_UINT32            timeoutUs,
  _ADB3_UINT32            commandLength,
  const _ADB3_UINT8*      pCommand,
  _ADB3_UINT32            responseLength,
  _ADB3_UINT8*            pResponse,
  _ADB3_UINT32*           pActualResponseLength);

typedef ADB3_STATUS ADB3_CALLING_CONVENTION FN_ADB3_GetAVR2Status(
  ADB3_HANDLE             hCard,
  ADB3_AVR2_STATUS*       pAVR2Status);

typedef const TCHAR* ADB3_CALLING_CONVENTION FN_ADB3_GetStatusString(
  ADB3_STATUS             code,
  _ADB3_BOOL              bShort);

typedef ADB3_STATUS ADB3_CALLING_CONVENTION FN_ADB3_GetUniqueId(
  ADB3_HANDLE             hCard,
  ADB3_UNIQUE_ID*         pUniqueId);

typedef ADB3_STATUS ADB3_CALLING_CONVENTION FN_ADB3_Open(
  unsigned int            index,
  _ADB3_BOOL              bPassive,
  _ADB3_UINT32            cooperativeLevel,
  ADB3_HANDLE*            phCard);

/* Structure to contain pointers to ADB3 API functions */
typedef struct _API_ADB3 {
  FN_ADB3_Close*           pfnADB3_Close;
  FN_ADB3_CommandAVR2*     pfnADB3_CommandAVR2;
  FN_ADB3_GetAVR2Status*   pfnADB3_GetAVR2Status;
  FN_ADB3_GetStatusString* pfnADB3_GetStatusString;
  FN_ADB3_GetUniqueId*     pfnADB3_GetUniqueId;
  FN_ADB3_Open*            pfnADB3_Open;
} API_ADB3;

struct _DeviceHandle {
  bool_t bValid;
  Avr2utilInterfaceType nInterfaceType;
  TCHAR* pszInterfaceName;
  bool_t bEnterServiceModeRequestAllowed;
  bool_t bExitServiceModeRequestAllowed;
  bool_t bAutoEnterServiceMode;
  bool_t bInServiceMode;
  unsigned int nCommandTimeoutUs;
  unsigned int nRebootDelayMs;
  union _Variant {
#if defined(AVR2_VIA_USB_SUPPORTED)
    struct {
      AVR2_HANDLE hDevice;
      API_AVR2 api;
    } avr2;
#endif
#if defined(AVR2_VIA_SMBUS_SUPPORTED)
    struct {
      AVR2S_HANDLE hDevice;
      API_AVR2S api;
    } avr2s;
#endif
#if defined(AVR2_VIA_IPMI_SUPPORTED)
    struct {
      AVR2IPMI_SESSION hSession;
      API_AVR2IPMI api;
    } avr2ipmi;
#endif
#if defined(AVR2_VIA_ADXDMA_SUPPORTED)
    struct {
      ADXDMA_HBC hBC;
      API_ADXDMA api;
    } adxdma;
#endif
    struct {
      ADB3_HANDLE hDevice;
      API_ADB3 api;
    } adb3;
  } variant;
};

/* Models that this utility can communicate with via PCIe */
struct _ADB3SupportedModels {
  ADMXRC3_MODEL_TYPE modelType;
  const TCHAR* pszModelName;
} g_adb3SupportedModels[] = {
  { ADMXRC3_MODEL_ADMXRCKU1,    _T("ADM-XRC-KU1") },
  { ADMXRC3_MODEL_ADMXRCKU1AT8, _T("ADM-XRC-KU1A-T8") },
  { ADMXRC3_MODEL_ADMPCIE8V3,   _T("ADM-PCIE-8V3") },
  { ADMXRC3_MODEL_ADMPCIE8K5,   _T("ADM-PCIE-8K5(-FH) / ADM-PCIE-UCC") }
};

#if !(defined(__VXWORKS__) || defined(__vxworks))

static int stringToUInt(
  const TCHAR* pString,
  unsigned int* pValue)
{
  if (_tcslen(pString) >= 2 && pString[0] == _T('0') && pString[1] == _T('x')) {
    return _stscanf_s(pString + 2, _T("%x"), pValue);
  } else {
    return _stscanf_s(pString, _T("%u"), pValue);
  }
}

#endif

#if defined(AVR2_VIA_USB_SUPPORTED)

# if defined(AVR2_LIBRARY_STATIC)

/* AVR2 API library was statically linked, so no need to check for its presence. */
static int
LoadAVR2Library(
  API_AVR2* pApi)
{
  pApi->pfnAVR2_Close = &AVR2_Close;
  pApi->pfnAVR2_Command = &AVR2_Command;
  pApi->pfnAVR2_GetStatusString = &AVR2_GetStatusString;
  pApi->pfnAVR2_OpenPort = &AVR2_OpenPort;
  pApi->pfnAVR2_SetPortBaud = &AVR2_SetPortBaud;

  return 0;
}

# else

static const char* g_pszAVR2APIFunctions[] = {
  "AVR2_Close",
  "AVR2_Command",
#   if defined(_UNICODE)
  "AVR2_GetStatusStringW",
  "AVR2_OpenPortW",
#   else
  "AVR2_GetStatusStringA",
  "AVR2_OpenPortA",
#   endif
  "AVR2_SetPortBaud"
};

#   if defined(_WIN32)
/* Windows - check for adavr2[d].dll using LoadLibrary(). If we can load it,
** we assume that subsequent delayed-loading of the DLL will succeed.
** This check is performance for Windows because if a delay-loaded DLL can't
** be found, the program crashes in a very unhelpful way, with no error
** message given to the user about why it crashed. */
static int
LoadAVR2Library(
  API_AVR2* pApi)
{
#     if defined(_DEBUG)
  static const char* pDLLName = "adavr2d.dll";
#     else
  static const char* pDLLName = "adavr2.dll";
#     endif
  HMODULE hModule;
  int i;
  FARPROC* ppFunctions;

  hModule = LoadLibraryA(pDLLName);
  if (NULL == hModule) {
    printf("*** Cannot find DLL '%s'. This can be corrected by installing the AVR2 System Monitor Driver, or copying '%s' to the same folder as this program.\n",
      pDLLName, pDLLName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (FARPROC*)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszAVR2APIFunctions); i++) {
    FARPROC pEntryPoint = GetProcAddress(hModule, g_pszAVR2APIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in DLL '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszAVR2APIFunctions[i], pDLLName);
      FreeLibrary(hModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   else
/* In Linux, we assume the shared object is available (we would not have even
** got this far if not). Linux gives a helpful error message when attempting
** to run a dynamically linked program for which one or more of its shared
** object references can't be resolved. */
static int
LoadAVR2Library(
  API_AVR2* pApi)
{
  static const char* pSoName = "libadavr2.so";
  void* pModule = NULL;
  int i;
  void** ppFunctions;

  pModule = dlopen("libadavr2.so", RTLD_NOW);
  if (NULL == pModule) {
    printf("*** Cannot find shared library '%s'. This can be corrected by installing the AVR2 System Monitor Driver, or copying '%s' to the same folder as this program.\n",
      pSoName, pSoName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (void**)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszAVR2APIFunctions); i++) {
    void* pEntryPoint = dlsym(pModule, g_pszAVR2APIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in shared library '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszAVR2APIFunctions[i], pSoName);
      dlclose(pModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   endif

# endif

#endif


#if defined(AVR2_VIA_SMBUS_SUPPORTED)

# if defined(AVR2S_LIBRARY_STATIC)

/* AVR2S API library was statically linked, so no need to check for its presence. */
static int
LoadAVR2SLibrary(
  API_AVR2S* pApi)
{
  pApi->pfnAVR2S_Close = &AVR2S_Close;
  pApi->pfnAVR2S_Command = &AVR2S_Command;
  pApi->pfnAVR2S_GetStatusString = &AVR2S_GetStatusString;
  pApi->pfnAVR2S_OpenDevice = &AVR2S_OpenDevice;

  return 0;
}

# else

static const char* g_pszAVR2SAPIFunctions[] = {
  "AVR2S_Close",
  "AVR2S_Command",
#   if defined(_UNICODE)
  "AVR2S_GetStatusStringW",
  "AVR2S_OpenDeviceW"
#   else
  "AVR2S_GetStatusStringA",
  "AVR2S_OpenDeviceA"
#   endif
};

#   if defined(_WIN32)
/* Windows - check for adavr2[d].dll using LoadLibrary(). If we can load it,
** we assume that subsequent delayed-loading of the DLL will succeed.
** This check is performance for Windows because if a delay-loaded DLL can't
** be found, the program crashes in a very unhelpful way, with no error
** message given to the user about why it crashed. */
static int
LoadAVR2SLibrary(
  API_AVR2S* pApi)
{
#     if defined(_DEBUG)
  static const char* pDLLName = "adavr2sd.dll";
#     else
  static const char* pDLLName = "adavr2s.dll";
#     endif
  HMODULE hModule;
  int i;
  FARPROC* ppFunctions;

  hModule = LoadLibraryA(pDLLName);
  if (NULL == hModule) {
    printf("*** Cannot find DLL '%s'. This can be corrected by installing the AVR2S System Monitor Driver, or copying '%s' to the same folder as this program.\n",
      pDLLName, pDLLName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (FARPROC*)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszAVR2SAPIFunctions); i++) {
    FARPROC pEntryPoint = GetProcAddress(hModule, g_pszAVR2SAPIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in DLL '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszAVR2SAPIFunctions[i], pDLLName);
      FreeLibrary(hModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   else
/* In Linux, we assume the shared object is available (we would not have even
** got this far if not). Linux gives a helpful error message when attempting
** to run a dynamically linked program for which one or more of its shared
** object references can't be resolved. */
static int
LoadAVR2SLibrary(
  API_AVR2S* pApi)
{
  static const char* pSoName = "libadavr2s.so";
  void* pModule = NULL;
  int i;
  void** ppFunctions;

  pModule = dlopen("libadavr2s.so", RTLD_NOW);
  if (NULL == pModule) {
    printf("*** Cannot find shared library '%s'. This can be corrected by installing the AVR2S System Monitor Driver, or copying '%s' to the same folder as this program.\n",
      pSoName, pSoName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (void**)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszAVR2SAPIFunctions); i++) {
    void* pEntryPoint = dlsym(pModule, g_pszAVR2SAPIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in shared library '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszAVR2SAPIFunctions[i], pSoName);
      dlclose(pModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   endif

# endif

#endif


#if defined(AVR2_VIA_IPMI_SUPPORTED)

# if defined(AVR2IPMI_LIBRARY_STATIC)

/* AVR2 IPMI library was statically linked, so no need to check for its presence. */
static int
LoadAVR2IPMILibrary(
  API_AVR2IPMI* pApi)
{
  pApi->pfnAVR2IPMI_CloseSession = &AVR2IPMI_CloseSession;
  pApi->pfnAVR2IPMI_Command = &AVR2IPMI_Command;
  pApi->pfnAVR2IPMI_GetBackends = &AVR2IPMI_GetBackends;
  pApi->pfnAVR2IPMI_GetErrorDetail = &AVR2IPMI_GetErrorDetail;
  pApi->pfnAVR2IPMI_GetStatusString = &AVR2IPMI_GetStatusString;
  pApi->pfnAVR2IPMI_OpenSession = &AVR2IPMI_OpenSession;
  pApi->pfnAVR2IPMI_ParseOptions = &AVR2IPMI_ParseOptions;

  return 0;
}

# else

static const char* g_pszAVR2IPMIAPIFunctions[] = {
  "AVR2IPMI_CloseSession",
  "AVR2IPMI_Command",
  "AVR2IPMI_OpenSession",
#   if defined(_UNICODE)
  "AVR2IPMI_GetBackendsW",
  "AVR2IPMI_GetErrorDetailW",
  "AVR2IPMI_GetStatusStringW",
  "AVR2IPMI_ParseOptionsW"
#   else
  "AVR2IPMI_GetBackendsA",
  "AVR2IPMI_GetErrorDetailA",
  "AVR2IPMI_GetStatusStringA",
  "AVR2IPMI_ParseOptionsA"
#   endif
};

#   if defined(_WIN32)
/* Windows - check for adavripmi[d].dll using LoadLibrary(). If we can load it,
** we assume that subsequent delayed-loading of the DLL will succeed.
** This check is performance for Windows because if a delay-loaded DLL can't
** be found, the program crashes in a very unhelpful way, with no error
** message given to the user about why it crashed. */
static int
LoadAVR2IPMILibrary(
  API_AVR2IPMI* pApi)
{
#     if defined(_DEBUG)
  static const char* pDLLName = "adavr2ipmid.dll";
#     else
  static const char* pDLLName = "adavr2ipmi.dll";
#     endif
  HMODULE hModule;
  int i;
  FARPROC* ppFunctions;

  hModule = LoadLibraryA(pDLLName);
  if (NULL == hModule) {
    printf("*** Cannot find DLL '%s'. This can be corrected by installing the AVR2IPMI System Monitor Driver, or copying '%s' to the same folder as this program.\n",
      pDLLName, pDLLName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (FARPROC*)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszAVR2IPMIAPIFunctions); i++) {
    FARPROC pEntryPoint = GetProcAddress(hModule, g_pszAVR2IPMIAPIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in DLL '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszAVR2IPMIAPIFunctions[i], pDLLName);
      FreeLibrary(hModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   else
/* In Linux, we assume the shared object is available (we would not have even
** got this far if not). Linux gives a helpful error message when attempting
** to run a dynamically linked program for which one or more of its shared
** object references can't be resolved. */
static int
LoadAVR2IPMILibrary(
  API_AVR2IPMI* pApi)
{
  static const char* pSoName = "libadavr2ipmi.so";
  void* pModule = NULL;
  int i;
  void** ppFunctions;

  pModule = dlopen("libadavr2s.so", RTLD_NOW);
  if (NULL == pModule) {
    printf("*** Cannot find shared library '%s'. This can be corrected by installing the AVR2S System Monitor Driver, or copying '%s' to the same folder as this program.\n",
      pSoName, pSoName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (void**)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszAVR2IPMIAPIFunctions); i++) {
    void* pEntryPoint = dlsym(pModule, g_pszAVR2IPMIAPIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in shared library '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszAVR2IPMIAPIFunctions[i], pSoName);
      dlclose(pModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   endif

# endif

#endif


#if defined(AVR2_VIA_ADXDMA_SUPPORTED)

# if defined(ADXDMA_LIBRARY_STATIC)

/* AVR2 IPMI library was statically linked, so no need to check for its presence. */
static int
LoadADXDMALibrary(
  API_ADXDMA* pApi)
{
  pApi->pfnADXDMA_CloseBC = &ADXDMA_CloseBC;
  pApi->pfnADXDMA_CommandUC = &ADXDMA_CommandUC;
  pApi->pfnADXDMA_GetStatusString = &ADXDMA_GetStatusString;
  pApi->pfnADXDMA_GetUCStatus = &ADXDMA_GetUCStatus;
  pApi->pfnADXDMA_OpenBC = &ADXDMA_OpenBC;

  return 0;
}

# else

static const char* g_pszADXDMAAPIFunctions[] = {
  "ADXDMA_CloseBC",
  "ADXDMA_CommandUC",
#   if defined(_UNICODE)
  "ADXDMA_GetStatusStringW",
#   else
  "ADXDMA_GetStatusStringA",
#   endif
  "ADXDMA_GetUCStatus",
  "ADXDMA_OpenBC"
};

#   if defined(_WIN32)
/* Windows - check for adxdma[d].dll using LoadLibrary(). If we can load it,
** we assume that subsequent delayed-loading of the DLL will succeed.
** This check is performance for Windows because if a delay-loaded DLL can't
** be found, the program crashes in a very unhelpful way, with no error
** message given to the user about why it crashed. */
static int
LoadADXDMALibrary(
  API_ADXDMA* pApi)
{
#     if defined(_DEBUG)
  static const char* pDLLName = "adxdmad.dll";
#     else
  static const char* pDLLName = "adxdma.dll";
#     endif
  HMODULE hModule;
  int i;
  FARPROC* ppFunctions;

  hModule = LoadLibraryA(pDLLName);
  if (NULL == hModule) {
    printf("*** Cannot find DLL '%s'. This can be corrected by installing the ADXDMA Driver, or copying '%s' to the same folder as this program.\n",
      pDLLName, pDLLName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (FARPROC*)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszADXDMAAPIFunctions); i++) {
    FARPROC pEntryPoint = GetProcAddress(hModule, g_pszADXDMAAPIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in DLL '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszADXDMAAPIFunctions[i], pDLLName);
      FreeLibrary(hModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   else
/* In Linux, we assume the shared object is available (we would not have even
** got this far if not). Linux gives a helpful error message when attempting
** to run a dynamically linked program for which one or more of its shared
** object references can't be resolved. */
static int
LoadADXDMALibrary(
  API_ADXDMA* pApi)
{
  static const char* pSoName = "libadxdma.so";
  void* pModule = NULL;
  int i;
  void** ppFunctions;

  pModule = dlopen(pSoName, RTLD_NOW);
  if (NULL == pModule) {
    printf("*** Cannot find shared library '%s'. This can be corrected by installing the AVR2S System Monitor Driver, or copying '%s' to the same folder as this program.\n",
      pSoName, pSoName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (void**)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszADXDMAAPIFunctions); i++) {
    void* pEntryPoint = dlsym(pModule, g_pszADXDMAAPIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in shared library '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszADXDMAAPIFunctions[i], pSoName);
      dlclose(pModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}
#   endif

# endif

#endif


# if defined(ADB3_LIBRARY_STATIC)

/* AVR2 API library was statically linked, so no need to check for its presence. */
static int
LoadADB3Library(
  API_ADB3* pApi)
{
  pApi->pfnADB3_Close = &ADB3_Close;
  pApi->pfnADB3_CommandAVR2 = &ADB3_CommandAVR2;
  pApi->pfnADB3_GetAVR2Status = &ADB3_GetAVR2Status;
  pApi->pfnADB3_GetStatusString = &ADB3_GetStatusString;
  pApi->pfnADB3_GetUniqueId = &ADB3_GetUniqueId;
  pApi->pfnADB3_Open = &ADB3_Open;

  return 0;
}

#else

static const char* g_pszADB3APIFunctions[] = {
  "ADB3_Close",
  "ADB3_CommandAVR2",
  "ADB3_GetAVR2Status",
# if defined(_UNICODE)
  "ADB3_GetStatusStringW",
# else
  "ADB3_GetStatusStringA",
# endif
  "ADB3_GetUniqueId",
  "ADB3_Open"
};

# if defined(_WIN32)

/* Windows - check for adb3[d].dll using LoadLibrary(). If we can load it,
** we assume that subsequent delayed-loading of the DLL will succeed.
** This check is performance for Windows because if a delay-loaded DLL can't
** be found, the program crashes in a very unhelpful way, with no error
** message given to the user about why it crashed. */
static int
LoadADB3Library(
  API_ADB3* pApi)
{
#   if defined(_DEBUG)
  static const char* pDLLName = "adb3d.dll";
#   else
  static const char* pDLLName = "adb3.dll";
#   endif
  HMODULE hModule;
  FARPROC* ppFunctions;
  int i;

  hModule = LoadLibraryA(pDLLName);
  if (NULL == hModule) {
    printf("*** Cannot find DLL '%s'. This can be corrected by installing the ADB3 Driver.\n", pDLLName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (FARPROC*)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszADB3APIFunctions); i++) {
    FARPROC pEntryPoint = GetProcAddress(hModule, g_pszADB3APIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in DLL '%s'. This may indicate that the DLL version is too old to run this program.\n",
        g_pszADB3APIFunctions[i], pDLLName);
      FreeLibrary(hModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}

# else

/* In Linux, we assume the shared object is available (we would not have even
** got this far if not). Linux gives a helpful error message when attempting
** to run a dynamically linked program for which one or more of its shared
** object references can't be resolved. */
static int
LoadADB3Library(
  API_ADB3* pApi)
{
  static const char* pSoName = "libadb3.so";
  void* pModule = NULL;
  int i;
  void** ppFunctions;

  pModule = dlopen("libadb3.so", RTLD_NOW);
  if (NULL == pModule) {
    printf("*** Cannot find shared library '%s'. This can be corrected by installing the ADB3 Driver.\n",
      pSoName);
    return EXIT_LIBRARY_NOT_FOUND;
  }

  ppFunctions = (void**)pApi;
  for (i = 0; i < ARRAY_LENGTH(g_pszADB3APIFunctions); i++) {
    void* pEntryPoint = dlsym(pModule, g_pszADB3APIFunctions[i]);
    if (NULL == pEntryPoint) {
      printf("*** Cannot find entry point '%s' in shared library '%s'. This may indicate that the shared library version is too old to run this program.\n",
        g_pszADB3APIFunctions[i], pSoName);
      dlclose(pModule);
      return EXIT_LIBRARY_NOT_FOUND;
    }
    *ppFunctions++ = pEntryPoint;
  }

  return 0;
}

# endif

#endif

DeviceHandle* allocDeviceHandle(
  void)
{
  return (DeviceHandle*)malloc(sizeof(DeviceHandle));
}

void freeDeviceHandle(
  DeviceHandle* pDevice)
{
  if (NULL != pDevice) {
    free(pDevice);
  }
}

static ADB3_STATUS
sampleAdb3OpenCard(
  unsigned int index,
  bool_t bByIndex,
  unsigned int serial,
  bool_t bBySerial,
  bool_t bReadOnly,
  ADB3_HANDLE* phDevice,
  API_ADB3* pApi)
{
  ADB3_STATUS status;
  ADB3_HANDLE hDevice;
  ADB3_UNIQUE_ID id;
  unsigned int i;

  if (bByIndex && bBySerial) {
    /* Attempt to open card with specified index and serial number */
    status = pApi->pfnADB3_Open(index, bReadOnly, 0, &hDevice);
    if (status != ADB3_SUCCESS) {
      return status;
    }
    status = pApi->pfnADB3_GetUniqueId(hDevice, &id);
    if (status != ADB3_SUCCESS) {
      return status;
    }
    if (id.SerialNumber == serial) {
      *phDevice = hDevice;
      return ADB3_SUCCESS;
    }
    return ADB3_DEVICE_NOT_FOUND;
  }

  if (bBySerial) {
    /* Attempt to open card with specified serial number */
    for (i = 0; i < 256; i++) {
      status = pApi->pfnADB3_Open(i, bReadOnly, 0, &hDevice);
      if (status != ADB3_SUCCESS) {
        if (status == ADB3_DEVICE_NOT_FOUND) {
          return status;
        }
        continue;
      }
      status = pApi->pfnADB3_GetUniqueId(hDevice, &id);
      if (status != ADB3_SUCCESS) {
        continue;
      }
      if (id.SerialNumber == serial) {
        *phDevice = hDevice;
        return ADB3_SUCCESS;
      }
      pApi->pfnADB3_Close(hDevice);
    }
    return ADB3_DEVICE_NOT_FOUND;
  } else {
    /* Attempt to open card with specified index */
    return pApi->pfnADB3_Open(index, bReadOnly, 0, phDevice);
  }
}

static void
dumpAvr2Command(
  const uint8_t* pBuffer,
  uint32_t length)
{
  uint32_t i;

  _tprintf(_T("DEBUG: AVR2 command[%lu] = { "), (unsigned long)length);
  for (i = 0; i < length; i++) {
    _tprintf(_T("%02X "), (unsigned int)pBuffer[i]);
  }
  _tprintf(_T("}\n"));
}

static void
dumpAvr2Response(
  const uint8_t* pBuffer,
  uint32_t expectedLength,
  uint32_t actualLength)
{
  uint32_t i, length;

  length = (actualLength < expectedLength) ? actualLength : expectedLength;
  _tprintf(_T("DEBUG: AVR2 response[%lu] = { "), (unsigned long)length);
  for (i = 0; i < length; i++) {
    _tprintf(_T("%02X "), (unsigned int)pBuffer[i]);
  }
  _tprintf(_T("}\n"));
  if (actualLength > expectedLength) {
    _tprintf(_T("DEBUG: %lu extra unexpected characters of actual AVR2 response were discarded.\n"),
      (unsigned long)(actualLength - expectedLength));
  }
}

int
avr2utilSendAvr2CommandEx(
  const DeviceHandle* pDevice,
  bool_t bVerbose,
  uint32_t timeoutUs,
  const uint8_t* pCommand,
  uint32_t commandLength,
   /* If nonzero, changes SMBUS chip address between command and response */
   /* Ignored if not communicating with uC via SMBUS or IPMI+SMBUS */
  uint8_t chipAddrOverride,
  const uint8_t expResponse[2],
  uint8_t* pResponse,
  uint32_t responseLength, /* 0 => Skip receiving response (SMBUS / IPMI+SMBUS only) */
  uint32_t* pActualLength)
{
  if (0 == timeoutUs) {
    timeoutUs = pDevice->nCommandTimeoutUs;
  }

  if (bVerbose) {
    dumpAvr2Command(pCommand, commandLength);
  }

  /* Prevent compiler whining about unused parameter when neither SMBUS nor IPMI+SMBUS compiled in */
  (void)chipAddrOverride;

  switch (pDevice->nInterfaceType) {
#if defined(AVR2_VIA_USB_SUPPORTED)
  case Avr2utilInterfaceAVR2: {
    AVR2_STATUS status;

    status = pDevice->variant.avr2.api.pfnAVR2_Command(
      pDevice->variant.avr2.hDevice, 0, timeoutUs, commandLength, pCommand, responseLength, pResponse, pActualLength);
    if (AVR2_SUCCESS != status) {
      _tprintf(_T("*** Failed to send command to AVR2 uC via USB: %s\n"), pDevice->variant.avr2.api.pfnAVR2_GetStatusString(status, TRUE));
      return EXIT_AVR2_COMMAND_ERROR;
    }

    break;
  }
#endif

#if defined(AVR2_VIA_SMBUS_SUPPORTED)
  case Avr2utilInterfaceAVR2S: {
    AVR2S_STATUS status;
    uint32_t flags = 0;

    status = pDevice->variant.avr2s.api.pfnAVR2S_Command(
      pDevice->variant.avr2s.hDevice, flags, timeoutUs, commandLength, pCommand, chipAddrOverride, responseLength, pResponse, pActualLength);
    if (AVR2S_SUCCESS != status) {
      _tprintf(_T("*** Failed to send command to AVR2 uC via SMBUS: %s\n"), pDevice->variant.avr2s.api.pfnAVR2S_GetStatusString(status, TRUE));
      return EXIT_AVR2_COMMAND_ERROR;
    }

    break;
  }
#endif

#if defined(AVR2_VIA_IPMI_SUPPORTED)
  case Avr2utilInterfaceAVR2IPMI: {
    AVR2IPMI_STATUS status;
    uint32_t flags = 0;

    status = pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_Command(
      pDevice->variant.avr2ipmi.hSession, flags, timeoutUs, commandLength, pCommand, chipAddrOverride, responseLength, pResponse, pActualLength);
    if (AVR2IPMI_SUCCESS != status) {
      _tprintf(_T("*** Failed to send command to AVR2 uC via IPMI: %s\n"), pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_GetStatusString(status, TRUE));
      return EXIT_AVR2_COMMAND_ERROR;
    }

    break;
  }
#endif

#if defined(AVR2_VIA_ADXDMA_SUPPORTED)
  case Avr2utilInterfaceADXDMA: {
    ADXDMA_STATUS status;

    status = pDevice->variant.adxdma.api.pfnADXDMA_CommandUC(
      pDevice->variant.adxdma.hBC, 0, timeoutUs, commandLength, pCommand, responseLength, pResponse, pActualLength);
    if (ADXDMA_SUCCESS != status) {
      _tprintf(_T("*** Failed to send command to uC: %s\n"), pDevice->variant.adxdma.api.pfnADXDMA_GetStatusString(status, TRUE));
      return EXIT_AVR2_COMMAND_ERROR;
    }

    break;
  }
#endif

  case Avr2utilInterfaceADB3: {
    ADB3_STATUS status;

    status = pDevice->variant.adb3.api.pfnADB3_CommandAVR2(
      pDevice->variant.adb3.hDevice, 0, timeoutUs, commandLength, pCommand, responseLength, pResponse, pActualLength);
    if (ADB3_SUCCESS != status) {
      _tprintf(_T("*** Failed to send command to AVR2 uC: %s\n"), pDevice->variant.adb3.api.pfnADB3_GetStatusString(status, TRUE));
      return EXIT_AVR2_COMMAND_ERROR;
    }

    break;
  }

  default:
    /* Should never get here */
    assert(FALSE);
    break;
  }

  if (bVerbose) {
    dumpAvr2Response(pResponse, responseLength, *pActualLength);
  }

  if (*pActualLength != responseLength) {
    _tprintf(_T("+++ Unexpected AVR2 response length %lu bytes; expected %lu bytes.\n"),
      (unsigned long)*pActualLength, (unsigned long)responseLength);
  }

  if (*pActualLength < 2) {
    _tprintf(_T("+++ AVR2 response too short to be valid.\n"));
    return EXIT_AVR2_SHORT_RESPONSE;
  } else {
    if (pResponse[0] != expResponse[0] || pResponse[1] != expResponse[1]) {
      _tprintf(_T("+++ Unexpected AVR2 response code { 0x%02X, 0x%02X }; expected { 0x%02X, 0x%02X }.\n"),
        (unsigned int)pResponse[0], (unsigned int)pResponse[1],
        (unsigned int)expResponse[0], (unsigned int)expResponse[1]);
    }
  }

  return 0;
}

int
avr2utilSendAvr2Command(
  const DeviceHandle* pDevice,
  bool_t bVerbose,
  uint32_t timeoutUs,
  const uint8_t* pCommand,
  uint32_t commandLength,
  const uint8_t expResponse[2],
  uint8_t* pResponse,
  uint32_t responseLength, /* 0 => Skip receiving response (SMBUS / IPMI+SMBUS only) */
  uint32_t* pActualLength)
{
  return avr2utilSendAvr2CommandEx(
    pDevice,
    bVerbose,
    timeoutUs,
    pCommand,
    commandLength,
    0,
    expResponse,
    pResponse,
    responseLength,
    pActualLength);
}

static int
modelSanityCheck(
  ADB3_HANDLE hDeviceAdb3,
  API_ADB3* pApi)
{
  ADB3_STATUS status;
  ADB3_UNIQUE_ID id;
  int i;

  status = pApi->pfnADB3_GetUniqueId(hDeviceAdb3, &id);
  if (ADB3_SUCCESS != status) {
    _tprintf(_T("*** Failed to get type of card: %s\n"), pApi->pfnADB3_GetStatusString(status, TRUE));
    return EXIT_UNSUPPORTED_MODEL;
  }

  for (i = 0; i < ARRAY_LENGTH(g_adb3SupportedModels); i++) {
    if (g_adb3SupportedModels[i].modelType == (ADMXRC3_MODEL_TYPE)id.Model) {
      return 0;
    }
  }

  _tprintf(_T("*** ADB3 Driver connection can be used only with the following models:\n"));
  _tprintf(_T("\n"));
  for (i = 0; i < ARRAY_LENGTH(g_adb3SupportedModels); i++) {
    _tprintf(_T("  o %s\n"), g_adb3SupportedModels[i].pszModelName);
  }

  return EXIT_UNSUPPORTED_MODEL;
}

#if !defined(_MSC_VER)
/* Emulate _tgetenv_s ("secure getenv") from MSVC++ */
static void
_tgetenv_s(
  size_t* pRequiredLength,
  TCHAR* pBuffer,
  size_t max,
  const TCHAR* pVariableName)
{
  TCHAR* pTmp;
  size_t length;

  pTmp = getenv(pVariableName);
  if (NULL == pTmp) {
    length = 0;
  } else {
    length = strlen(pTmp) + 1;
  }
  if (NULL != pRequiredLength) {
    /* Not found */
    *pRequiredLength = length;
  }
  if (NULL != pBuffer) {
    if (max) {
      max--; /* subtract 1 for NUL terminator */
      if (length > max) {
        length = max;
      }
      if (length) {
        memcpy(pBuffer, pTmp, length * sizeof(TCHAR));
      }
      pBuffer[length] = '\0';
    }
  }
}
#endif

void
avr2utilCheckEnvironment(
  DeviceHandle* pDevice)
{
  unsigned int flags = 0;
  size_t reqLen = 0;
  TCHAR buf[16];

  _tgetenv_s(&reqLen, buf, ARRAY_LENGTH(buf), _T("AVR2UTIL_FLAGS"));
  if (0 != reqLen) {
    if (1 == _stscanf_s(buf, _T("%u"), &flags)) {
      if (flags & AVR2UTIL_FLAG_EN_ENTER_SM) {
        pDevice->bEnterServiceModeRequestAllowed = TRUE;
      }
      if (flags & AVR2UTIL_FLAG_EN_EXIT_SM) {
        pDevice->bExitServiceModeRequestAllowed = TRUE;
      }
      if (flags & AVR2UTIL_FLAG_EN_AUTO_SM) {
        pDevice->bAutoEnterServiceMode = TRUE;
      }
      if (flags & AVR2UTIL_FLAG_DIS_ENTER_SM) {
        pDevice->bEnterServiceModeRequestAllowed = FALSE;
      }
      if (flags & AVR2UTIL_FLAG_DIS_EXIT_SM) {
        pDevice->bExitServiceModeRequestAllowed = FALSE;
      }
      if (flags & AVR2UTIL_FLAG_DIS_AUTO_SM) {
        pDevice->bAutoEnterServiceMode = FALSE;
      }
    }
  }
}

void
avr2utilCloseDevice(
  DeviceHandle* pDevice)
{
  if (NULL != pDevice) {
    if (pDevice->bValid) {
      switch (pDevice->nInterfaceType) {
  #if defined(AVR2_VIA_USB_SUPPORTED)
      case Avr2utilInterfaceAVR2:
        pDevice->variant.avr2.api.pfnAVR2_Close(pDevice->variant.avr2.hDevice);
        break;
  #endif

  #if defined(AVR2_VIA_SMBUS_SUPPORTED)
      case Avr2utilInterfaceAVR2S:
        pDevice->variant.avr2s.api.pfnAVR2S_Close(pDevice->variant.avr2s.hDevice);
        break;
  #endif

  #if defined(AVR2_VIA_IPMI_SUPPORTED)
      case Avr2utilInterfaceAVR2IPMI:
        pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_CloseSession(pDevice->variant.avr2ipmi.hSession);
        break;
  #endif

      case Avr2utilInterfaceADB3:
        pDevice->variant.adb3.api.pfnADB3_Close(pDevice->variant.adb3.hDevice);
        break;

  #if defined(AVR2_VIA_ADXDMA_SUPPORTED)
      case Avr2utilInterfaceADXDMA:
        pDevice->variant.adxdma.api.pfnADXDMA_CloseBC(pDevice->variant.adxdma.hBC);
        break;
  #endif

      default:
        /* Should never get here */
        assert(FALSE);
        break;
      }

      pDevice->bValid = FALSE;
    }

    freeDeviceHandle(pDevice);
  }
}

int
avr2utilOpenDeviceADB3(
  unsigned int index,
  bool_t bIndexSpecified,
  unsigned int serial,
  bool_t bSerialSpecified,
  bool_t bVerbose,
  DeviceHandle** ppDevice)
{
  DeviceHandle* pDevice = NULL;
  ADB3_HANDLE hDeviceAdb3 = ADB3_HANDLE_INVALID_VALUE;
  ADB3_STATUS status;
  int ret = 0;
  
  pDevice = allocDeviceHandle();
  if (NULL == pDevice) {
    ret = EXIT_ALLOCATION_FAILED;
    goto done;
  }

  ret = LoadADB3Library(&pDevice->variant.adb3.api);
  if (ret) {
    goto done;
  }

  status = sampleAdb3OpenCard(index, bIndexSpecified, serial, bSerialSpecified, FALSE, &hDeviceAdb3, &pDevice->variant.adb3.api);
  if (status != ADB3_SUCCESS) {
    _tprintf(_T("*** Failed to open card: %s\n"), pDevice->variant.adb3.api.pfnADB3_GetStatusString(status, TRUE));
    ret = EXIT_DEVICE_OPEN_ERROR;
    goto done;
  }
  pDevice->variant.adb3.hDevice = hDeviceAdb3;
  pDevice->nInterfaceType = Avr2utilInterfaceADB3;
  pDevice->pszInterfaceName = _T("ADB3 Driver (PCIe)");
  /* Entering / exiting Service Mode via PCIe is fully supported. */
  pDevice->bEnterServiceModeRequestAllowed = TRUE;
  pDevice->bExitServiceModeRequestAllowed = TRUE;
  pDevice->bAutoEnterServiceMode = TRUE;
  pDevice->bValid = TRUE;
  pDevice->nCommandTimeoutUs = DEFAULT_COMMAND_TIMEOUT_US;
  pDevice->nRebootDelayMs = UC_ADB3_REBOOT_DELAY_MS;

  ret = modelSanityCheck(pDevice->variant.adb3.hDevice, &pDevice->variant.adb3.api);
  if (ret) {
    goto done;
  }

  ret = avr2utilDetermineServiceMode(pDevice, bVerbose);
  if (ret) {
    goto done;
  }

done:
  if (ret) {
    avr2utilCloseDevice(pDevice);
  } else {
    *ppDevice = pDevice;
  }

  return ret;    
}

#if !(defined(__VXWORKS__) || defined(__vxworks))

int
avr2utilOpenDeviceADXDMA(
  unsigned int index,
  bool_t bVerbose,
  DeviceHandle** ppDevice)
{
#if defined(AVR2_VIA_ADXDMA_SUPPORTED)

  int ret = 0;
  DeviceHandle* pDevice = NULL;
  ADXDMA_HBC hBC = ADXDMA_NULL_HBC;
  ADXDMA_STATUS status;
  
  pDevice = allocDeviceHandle();
  if (NULL == pDevice) {
    ret = EXIT_ALLOCATION_FAILED;
    goto done;
  }

  ret = LoadADXDMALibrary(&pDevice->variant.adxdma.api);
  if (ret) {
    goto done;
  }

  status = pDevice->variant.adxdma.api.pfnADXDMA_OpenBC(ADXDMA_NULL_HDEVICE, index, FALSE, &hBC);
  if (status != ADXDMA_SUCCESS) {
    _tprintf(_T("*** Failed to open device: %s\n"), pDevice->variant.adxdma.api.pfnADXDMA_GetStatusString(status, TRUE));
    ret = EXIT_DEVICE_OPEN_ERROR;
    goto done;
  }

  pDevice->variant.adxdma.hBC = hBC;
  pDevice->nInterfaceType = Avr2utilInterfaceADXDMA;
  pDevice->pszInterfaceName = _T("ADXDMA Driver (PCIe)");
  /* Entering / exiting Service Mode via PCIe is fully supported. */
  pDevice->bEnterServiceModeRequestAllowed = TRUE;
  pDevice->bExitServiceModeRequestAllowed = TRUE;
  pDevice->bAutoEnterServiceMode = TRUE;
  pDevice->bValid = TRUE;
  pDevice->nCommandTimeoutUs = DEFAULT_COMMAND_TIMEOUT_US;
  pDevice->nRebootDelayMs = UC_ADXDMA_REBOOT_DELAY_MS;

  ret = avr2utilDetermineServiceMode(pDevice, bVerbose);
  if (ret) {
    goto done;
  }

done:
  if (ret) {
    avr2utilCloseDevice(pDevice);
  } else {
    *ppDevice = pDevice;
  }

  return ret;    

#else

# if defined(_WIN32)

  _tprintf(_T("*** Access to the AVR2 uC via the ADXDMA Driver is currently not supported for the Windows operating system.\n"));

# elif (defined(__VXWORKS__) || defined(__vxworks))

  _tprintf(_T("*** Access to the AVR2 uC via the ADXDMA Driver is currently not supported for the VxWorks RTOS.\n"));

# else

  _tprintf(_T("*** Access to the AVR2 uC via the ADXDMA Driver is currently not supported for the GNU/Linux operating system.\n"));

# endif

  return EXIT_USB_NOT_SUPPORTED;

#endif
}

#define DRAWING_NUM_ADMVPX39Z2    (1323)
#define DRAWING_NUM_ADMXRC9R1     (1353)
#define DRAWING_NUM_ADMXRC9Z1     (1437)

int
avr2utilOpenDevicePsuart(
  const TCHAR* pConnectionString,
  bool_t bVerbose,
  DeviceHandle** ppDevice)
{
  int ret = 0;
  DeviceHandle* pDevice = NULL;
  AVR2_HANDLE hDeviceAvr2Usb = AVR2_HANDLE_INVALID_VALUE;
  AVR2_STATUS status;
  uint32_t productId;
  TCHAR szDeviceName[32], szBaudRate[32];
  bool_t bBadConnectionString = FALSE;
  bool_t bSetBaudRate = FALSE;
  uint8_t vpdBuffer[256]; /* Request VPD command requires buffer of 256 bytes */
  uint32_t drawingNumber;
  const TCHAR* p, * pEnd, * q;
  size_t n;
  unsigned int nElement, nBaudRate = 0;
  int nConverted;

  /*
  ** Parse connection string in form "<COM port>[,baud]" or "<device>[,baud]"
  ** In Linux, <device> is typically of the form "/dev/tty<something>"
  */

  nElement = 0;
  p = pConnectionString;
  pEnd = p + _tcslen(pConnectionString);

  memset(szDeviceName, 0, sizeof(szDeviceName));
  memset(szBaudRate, 0, sizeof(szBaudRate));

  while (p < pEnd) {
    q = _tcschr(p, _T(','));
    if (NULL == q) {
      q = pEnd;
    }

    n = q - p;

    if (n < 1) {
      /* Too short */
      bBadConnectionString = TRUE;
      break;
    }

    switch (nElement) {
    case 0:
      if (n > ARRAY_LENGTH(szDeviceName)) {
        n = ARRAY_LENGTH(szDeviceName);
      }
      memcpy(szDeviceName, p, n * sizeof(TCHAR));
      szDeviceName[ARRAY_LENGTH(szDeviceName) - 1] = _T('\0');
      break;

    case 1:
      if (n > ARRAY_LENGTH(szBaudRate)) {
        n = ARRAY_LENGTH(szBaudRate);
      }
      memcpy(szBaudRate, p, n * sizeof(TCHAR));
      szBaudRate[ARRAY_LENGTH(szBaudRate) - 1] = _T('\0');
      bSetBaudRate = TRUE;
      break;

    default:
      break;
    }

    if (nElement >= 2) {
      _tprintf(_T("+++ Ignoring unexpected extra elements of '-psuart' connection string: '%s'\n"), p);
      break;
    }

    nElement++;

    /* Skip to next field, if it exists */
    p = q + 1;
  }

  if (0 == nElement || bBadConnectionString) {
#ifdef _WIN32
    _tprintf(_T("*** Bad connection string '%s'; must be of form 'COM<n>[,baud]'\n"), pConnectionString);
#else
    _tprintf(_T("*** Bad connection string '%s'; must be of form '/dev/<device>[,baud]'\n"), pConnectionString);
#endif
    ret = EXIT_DEVICE_OPEN_ERROR;
    goto done;
  }

  if (bSetBaudRate) {
    nConverted = stringToUInt(szBaudRate, &nBaudRate);
    if (1 != nConverted) {
      _tprintf(_T("*** Baud rate value '%s' from '-psuart' connection string not recognized as a number.\n"), szBaudRate);
      ret = EXIT_INVALID_BAUD_RATE;
      goto done;
    }
  }

  pDevice = allocDeviceHandle();
  if (NULL == pDevice) {
    ret = EXIT_ALLOCATION_FAILED;
    goto done;
  }

  ret = LoadAVR2Library(&pDevice->variant.avr2.api);
  if (ret) {
    goto done;
  }

  status = pDevice->variant.avr2.api.pfnAVR2_OpenPort(szDeviceName, &hDeviceAvr2Usb);
  if (AVR2_SUCCESS != status) {
    _tprintf(_T("*** Failed to open port '%s': %s\n"), szDeviceName, pDevice->variant.avr2.api.pfnAVR2_GetStatusString(status, TRUE));
    ret = EXIT_DEVICE_OPEN_ERROR;
    goto done;
  }

  if (bSetBaudRate) {
    status = pDevice->variant.avr2.api.pfnAVR2_SetPortBaud(hDeviceAvr2Usb, nBaudRate);
    if (AVR2_SUCCESS != status) {
      _tprintf(_T("*** Failed to set port '%s' baud rate to %u: %s\n"),
        szDeviceName, nBaudRate, pDevice->variant.avr2.api.pfnAVR2_GetStatusString(status, TRUE));
      ret = EXIT_SET_BAUD_FAILED;
      goto done;
    }
  }

  pDevice->variant.avr2.hDevice = hDeviceAvr2Usb;
  pDevice->nInterfaceType = Avr2utilInterfaceAVR2;
  pDevice->pszInterfaceName = _T("PS UART");
  /* Assume entering Service Mode via UART serial connection is supported; will correct later. */
  pDevice->bEnterServiceModeRequestAllowed = TRUE;
  pDevice->bExitServiceModeRequestAllowed = TRUE;
  pDevice->bAutoEnterServiceMode = TRUE;
  pDevice->bValid = TRUE;
  pDevice->nCommandTimeoutUs = DEFAULT_COMMAND_TIMEOUT_US;
  pDevice->nRebootDelayMs = UC_PSUART_REBOOT_DELAY_MS;

  ret = avr2utilProductIdCommand(pDevice, bVerbose, TRUE, &productId);
  if (0 != ret) {
    _tprintf(_T("*** Failed to get AVR2 firmware Product ID.\n"));
    goto done;
  }

  if (productId == DRAWING_NUM_BOOTMAN2 || productId == DRAWING_NUM_BOOTMAN2_ALT1 || productId == DRAWING_NUM_BOOTMAN2_ALT2) {
    pDevice->bInServiceMode = TRUE;
  } else if (productId == DRAWING_NUM_BOARDMAN2 || productId == DRAWING_NUM_BOARDMAN2_ALT1 || productId == DRAWING_NUM_BOARDMAN2_ALT2) {
    pDevice->bInServiceMode = FALSE;
  } else {
    /* When connecting to uC via true UART serial connection, we can't easily check that we are talking a model whose
    ** Flash layout we know about. To protect against future models having a different Flash address map, we abort
    ** if we see a Product ID that we do not recognize.
    ** If the Flash address map is changed in a future model, it will have a different Product ID. */
    _tprintf(_T("*** AVR2 firmware Product ID %lu(0x%lX) not recognized. Aborting as a precaution against firmware corruption.\n"),
       (unsigned long)productId, (unsigned long)productId);
    ret = EXIT_UNRECOGNIZED_PRODUCTID;
    goto done;
  }

  memset(vpdBuffer, 0, sizeof(vpdBuffer));

  if (pDevice->bInServiceMode) {
    /* In Service Mode, get VPD by reading AVR Flash */
    ret = avr2utilReadApplicationFlash(pDevice, bVerbose, VPD_BASE_ADDR, vpdBuffer, VPD_LENGTH_LIMIT);
  } else {
    /* In Application Mode, request VPD using command */
    ret = avr2utilRequestVpd(pDevice, bVerbose, vpdBuffer);
  }

  /*
  ** Check that board we are talking to is one of the Zynq Ultrascale
  ** boards, because this has a PS UART connection to the microcontroller
  ** and thus switching in and out of Service Mode over the UART connection
  ** will work.
  */
  drawingNumber = GET_VPD_VAL32(&vpdBuffer[4]);
  switch (drawingNumber) {
  case DRAWING_NUM_ADMVPX39Z2:
  case DRAWING_NUM_ADMXRC9R1:
  case DRAWING_NUM_ADMXRC9Z1:
    /* OK to switch in and out of Service Mode over UART connection */
    break;

  default:
    /*
    ** Other board: this is very likely to be a USB UART connection, and
    ** switching in and out of Service Mode over this connection will not
    ** work because the USB device will disappear any time the
    ** microcontroller reboots in order to switch into Service Mode.
    */
    pDevice->bEnterServiceModeRequestAllowed = FALSE;
    pDevice->bExitServiceModeRequestAllowed = FALSE;
    pDevice->bAutoEnterServiceMode = FALSE;
    break;
  }

done:
  if (ret) {
    avr2utilCloseDevice(pDevice);
  } else {
    *ppDevice = pDevice;
  }

  return ret;    
}

#endif

int
avr2utilOpenDeviceUSBCom(
  const TCHAR* pDeviceName,
  bool_t bVerbose,
  DeviceHandle** ppDevice)
{
#if defined(AVR2_VIA_USB_SUPPORTED)

  int ret = 0;
  DeviceHandle* pDevice = NULL;
  AVR2_HANDLE hDeviceAvr2Usb = AVR2_HANDLE_INVALID_VALUE;
  AVR2_STATUS status;
  uint32_t productId;
  
  pDevice = allocDeviceHandle();
  if (NULL == pDevice) {
    ret = EXIT_ALLOCATION_FAILED;
    goto done;
  }

  ret = LoadAVR2Library(&pDevice->variant.avr2.api);
  if (ret) {
    goto done;
  }

  status = pDevice->variant.avr2.api.pfnAVR2_OpenPort(pDeviceName, &hDeviceAvr2Usb);
  if (AVR2_SUCCESS != status) {
    _tprintf(_T("*** Failed to open port '%s': %s\n"), pDeviceName, pDevice->variant.avr2.api.pfnAVR2_GetStatusString(status, TRUE));
    ret = EXIT_DEVICE_OPEN_ERROR;
    goto done;
  }

  pDevice->variant.avr2.hDevice = hDeviceAvr2Usb;
  pDevice->nInterfaceType = Avr2utilInterfaceAVR2;
  pDevice->pszInterfaceName = _T("USB");
  /* Assume entering Service Mode via USB not supported; will correct later. */
  pDevice->bEnterServiceModeRequestAllowed = FALSE;
  pDevice->bExitServiceModeRequestAllowed = FALSE;
  pDevice->bAutoEnterServiceMode = FALSE;
  pDevice->bValid = TRUE;
  pDevice->nCommandTimeoutUs = DEFAULT_COMMAND_TIMEOUT_US;
  pDevice->nRebootDelayMs = UC_USBCOM_REBOOT_DELAY_MS;

  ret = avr2utilProductIdCommand(pDevice, bVerbose, TRUE, &productId);
  if (0 != ret) {
    _tprintf(_T("*** Failed to get AVR2 firmware Product ID.\n"));
    goto done;
  }
  if (productId == DRAWING_NUM_BOOTMAN2 || productId == DRAWING_NUM_BOOTMAN2_ALT1 || productId == DRAWING_NUM_BOOTMAN2_ALT2) {
    pDevice->bInServiceMode = TRUE;
    /* If in Service Mode and successfully talking to uC over USB, BootMan2 must already support USB communication. */
    pDevice->bEnterServiceModeRequestAllowed = TRUE;
    pDevice->bExitServiceModeRequestAllowed = TRUE;
  } else if (productId == DRAWING_NUM_BOARDMAN2 || productId == DRAWING_NUM_BOARDMAN2_ALT1 || productId == DRAWING_NUM_BOARDMAN2_ALT2) {
    pDevice->bInServiceMode = FALSE;
  } else {
    /* When connecting to uC via USB serial connection, we can't easily check that we are talking a model whose
    ** Flash layout we know about. To protect against future models having a different Flash address map, we abort
    ** if we see a Product ID that we do not recognize.
    ** If the Flash address map is changed in a future model, it will have a different Product ID. */
    _tprintf(_T("*** AVR2 firmware Product ID %lu(0x%lX) not recognized. Aborting as a precaution against firmware corruption.\n"),
       (unsigned long)productId, (unsigned long)productId);
    ret = EXIT_UNRECOGNIZED_PRODUCTID;
    goto done;
  }

done:
  if (ret) {
    avr2utilCloseDevice(pDevice);
  } else {
    *ppDevice = pDevice;
  }

  return ret;    

#else

# if defined(_WIN32)

  UNREFERENCED_PARAMETER(pDeviceName);
  UNREFERENCED_PARAMETER(bVerbose);
  UNREFERENCED_PARAMETER(pDevice);

  _tprintf(_T("*** Access to the AVR2 uC via USB is currently not supported for the Windows operating system.\n"));

# elif (defined(__VXWORKS__) || defined(__vxworks))
  
  _tprintf(_T("*** Access to the AVR2 uC via USB is currently not supported for the VxWorks RTOS.\n"));

# else

  _tprintf(_T("*** Access to the AVR2 uC via USB is currently not supported for the GNU/Linux operating system.\n"));

# endif

  return EXIT_USB_NOT_SUPPORTED;

#endif
}

#if !(defined(__VXWORKS__) || defined(__vxworks))

int
avr2utilOpenDeviceSMBUS(
  const TCHAR* pDeviceName,
  uint8_t chipAddr,
  bool_t bVerbose,
  DeviceHandle** ppDevice)
{
#if defined(AVR2_VIA_SMBUS_SUPPORTED)

  int ret = 0;
  DeviceHandle* pDevice = NULL;
  AVR2S_HANDLE hDeviceAvr2Smbus = AVR2S_HANDLE_INVALID_VALUE;
  AVR2S_STATUS status;
  uint32_t productId;

  pDevice = allocDeviceHandle();
  if (NULL == pDevice) {
    ret = EXIT_ALLOCATION_FAILED;
    goto done;
  }

  ret = LoadAVR2SLibrary(&pDevice->variant.avr2s.api);
  if (ret) {
    goto done;
  }

  status = pDevice->variant.avr2s.api.pfnAVR2S_OpenDevice(pDeviceName, chipAddr, &hDeviceAvr2Smbus);
  if (AVR2S_SUCCESS != status) {
    _tprintf(_T("*** Failed to open I2C / SMBUS device '%s': %s\n"), pDeviceName, pDevice->variant.avr2s.api.pfnAVR2S_GetStatusString(status, TRUE));
    ret = EXIT_DEVICE_OPEN_ERROR;
    goto done;
  }

  pDevice->variant.avr2s.hDevice = hDeviceAvr2Smbus;
  pDevice->nInterfaceType = Avr2utilInterfaceAVR2S;
  pDevice->pszInterfaceName = _T("SMBUS");
  /* Entering / exiting Service Mode via SMBUS is fully supported. */
  pDevice->bEnterServiceModeRequestAllowed = TRUE;
  pDevice->bExitServiceModeRequestAllowed = TRUE;
  pDevice->bAutoEnterServiceMode = TRUE;
  pDevice->bValid = TRUE;
  pDevice->nCommandTimeoutUs = DEFAULT_COMMAND_TIMEOUT_US;
  pDevice->nRebootDelayMs = UC_SMBUS_REBOOT_DELAY_MS;

  ret = avr2utilProductIdCommand(pDevice, bVerbose, TRUE, &productId);
  if (0 != ret) {
    _tprintf(_T("*** Failed to get AVR2 firmware Product ID.\n"));
    goto done;
  }
  if (productId == DRAWING_NUM_BOOTMAN2 || productId == DRAWING_NUM_BOOTMAN2_ALT1 || productId == DRAWING_NUM_BOOTMAN2_ALT2) {
    pDevice->bInServiceMode = TRUE;
  } else if (productId == DRAWING_NUM_BOARDMAN2 || productId == DRAWING_NUM_BOARDMAN2_ALT1 || productId == DRAWING_NUM_BOARDMAN2_ALT2) {
    pDevice->bInServiceMode = FALSE;
  } else {
    /* When connecting to uC via USB serial connection, we can't easily check that we are talking a model whose
    ** Flash layout we know about. To protect against future models having a different Flash address map, we abort
    ** if we see a Product ID that we do not recognize.
    ** If the Flash address map is changed in a future model, it will have a different Product ID. */
    _tprintf(_T("*** AVR2 firmware Product ID %lu(0x%lX) not recognized. Aborting as a precaution against firmware corruption.\n"),
       (unsigned long)productId, (unsigned long)productId);
    ret = EXIT_UNRECOGNIZED_PRODUCTID;
    goto done;
  }

done:
  if (ret) {
    avr2utilCloseDevice(pDevice);
  } else {
    *ppDevice = pDevice;
  }

  return ret;

#else

# if defined(_WIN32)

  UNREFERENCED_PARAMETER(pDeviceName);
  UNREFERENCED_PARAMETER(chipAddr);
  UNREFERENCED_PARAMETER(bVerbose);
  UNREFERENCED_PARAMETER(ppDevice);

  _tprintf(_T("*** Access to the AVR2 uC via SMBUS is currently not supported for the Windows operating system.\n"));

# elif (defined(__VXWORKS__) || defined(__vxworks))

  _tprintf(_T("*** Access to the AVR2 uC via SMBUS is currently not supported for the VxWorks RTOS.\n"));

# else

  _tprintf(_T("*** Access to the AVR2 uC via SMBUS is currently not supported for the GNU/Linux operating system.\n"));

# endif

  return EXIT_SMBUS_NOT_SUPPORTED;

#endif
}

int
avr2utilOpenDeviceIPMI(
  const TCHAR* pDeviceName,
  bool_t bVerbose,
  DeviceHandle** ppDevice)
{
#if defined(AVR2_VIA_IPMI_SUPPORTED)

  int ret = 0;
  DeviceHandle* pDevice = NULL;
  AVR2IPMI_SESSION hSession = AVR2IPMI_SESSION_INVALID_VALUE;
  AVR2IPMI_OPTIONS sessionOptions;
  AVR2IPMI_ERROR_DETAIL errorDetail;
  AVR2IPMI_STATUS status;
  uint32_t productId;

  pDevice = allocDeviceHandle();
  if (NULL == pDevice) {
    ret = EXIT_ALLOCATION_FAILED;
    goto done;
  }

  ret = LoadAVR2IPMILibrary(&pDevice->variant.avr2ipmi.api);
  if (ret) {
    goto done;
  }

  status = pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_ParseOptions(pDeviceName, &sessionOptions);
  if (AVR2IPMI_SUCCESS != status) {
    _tprintf(_T("*** Failed to parse IPMI session options '%s': %s\n"), pDeviceName, pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_GetStatusString(status, TRUE));
    ret = EXIT_BAD_IPMI_OPTIONS;
    goto done;
  }

  status = pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_OpenSession(&sessionOptions, &hSession);
  if (AVR2IPMI_SUCCESS != status) {
    pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_GetErrorDetail(&errorDetail);
    _tprintf(_T("*** Failed to open IPMI session with options '%s': %s\n"), pDeviceName, pDevice->variant.avr2ipmi.api.pfnAVR2IPMI_GetStatusString(status, TRUE));
    _tprintf(_T("    Error detail is: %s\n"), errorDetail.Description);
    ret = EXIT_DEVICE_OPEN_ERROR;
    goto done;
  }

  pDevice->variant.avr2ipmi.hSession = hSession;
  pDevice->nInterfaceType = Avr2utilInterfaceAVR2IPMI;
  pDevice->pszInterfaceName = _T("IPMI");
  /* Entering / exiting Service Mode via IPMI (using SMBUS from BMC) is fully supported. */
  pDevice->bEnterServiceModeRequestAllowed = TRUE;
  pDevice->bExitServiceModeRequestAllowed = TRUE;
  pDevice->bAutoEnterServiceMode = TRUE;
  pDevice->bValid = TRUE;
  pDevice->nCommandTimeoutUs = DEFAULT_COMMAND_TIMEOUT_US;

  ret = avr2utilProductIdCommand(pDevice, bVerbose, TRUE, &productId);
  if (0 != ret) {
    _tprintf(_T("*** Failed to get AVR2 firmware Product ID.\n"));
    goto done;
  }
  if (productId == DRAWING_NUM_BOOTMAN2 || productId == DRAWING_NUM_BOOTMAN2_ALT1 || productId == DRAWING_NUM_BOOTMAN2_ALT2) {
    pDevice->bInServiceMode = TRUE;
  } else if (productId == DRAWING_NUM_BOARDMAN2 || productId == DRAWING_NUM_BOARDMAN2_ALT1 || productId == DRAWING_NUM_BOARDMAN2_ALT2) {
    pDevice->bInServiceMode = FALSE;
  } else {
    /* When connecting to uC via USB serial connection, we can't easily check that we are talking a model whose
    ** Flash layout we know about. To protect against future models having a different Flash address map, we abort
    ** if we see a Product ID that we do not recognize.
    ** If the Flash address map is changed in a future model, it will have a different Product ID. */
    _tprintf(_T("*** AVR2 firmware Product ID %lu(0x%lX) not recognized. Aborting as a precaution against firmware corruption.\n"),
       (unsigned long)productId, (unsigned long)productId);
    ret = EXIT_UNRECOGNIZED_PRODUCTID;
    goto done;
  }

done:
  if (ret) {
    avr2utilCloseDevice(pDevice);
  } else {
    *ppDevice = pDevice;
  }

  return ret;

#else

# if defined(_WIN32)

  UNREFERENCED_PARAMETER(pDeviceName);
  UNREFERENCED_PARAMETER(bVerbose);
  UNREFERENCED_PARAMETER(ppDevice);

  _tprintf(_T("*** Access to the AVR2 uC via IPMI is currently not supported for the Windows operating system.\n"));

# elif (defined(__VXWORKS__) || defined(__vxworks))

  _tprintf(_T("*** Access to the AVR2 uC via IPMI is currently not supported for the VxWorks RTOS.\n"));

# else

  _tprintf(_T("*** Access to the AVR2 uC via IPMI is currently not supported for the GNU/Linux operating system.\n"));

# endif

  return EXIT_IPMI_NOT_SUPPORTED;

#endif
}

#endif

Avr2utilInterfaceType
avr2utilGetInterfaceType(
  const DeviceHandle* pDevice)
{
  return pDevice->nInterfaceType;
}

const TCHAR*
avr2utilGetInterfaceName(
  const DeviceHandle* pDevice)
{
  return pDevice->pszInterfaceName;
}

void
avr2utilSetCommandTimeoutUs(
  DeviceHandle* pDevice,
  unsigned int timeoutMs)
{
  pDevice->nCommandTimeoutUs = timeoutMs * 1000;
}

void
avr2utilRebootDelay(
  DeviceHandle* pDevice)
{
  sleepMs(pDevice->nRebootDelayMs);
}

int
avr2utilDetermineServiceMode(
  DeviceHandle* pDevice,
  bool_t bVerbose)
{
  uint32_t nProductId;
  int ret = 0;

  ret = avr2utilProductIdCommand(pDevice, bVerbose, TRUE, &nProductId);
  if (0 != ret) {
    _tprintf(_T("*** Failed to get AVR2 firmware Product ID.\n"));
    goto done;
  }

  if (nProductId == DRAWING_NUM_BOOTMAN2 || nProductId == DRAWING_NUM_BOOTMAN2_ALT1 || nProductId == DRAWING_NUM_BOOTMAN2_ALT2) {
    pDevice->bInServiceMode = TRUE;
  } else if (nProductId == DRAWING_NUM_BOARDMAN2 || nProductId == DRAWING_NUM_BOARDMAN2_ALT1 || nProductId == DRAWING_NUM_BOARDMAN2_ALT2) {
    pDevice->bInServiceMode = FALSE;
  } else {
    /* To protect against future models having a different Flash address map, we abort
    ** if we see a Product ID that we do not recognize.
    ** If the Flash address map is changed in a future model, it will have a different Product ID. */
    _tprintf(_T("*** AVR2 firmware Product ID %u(0x%X) not recognized. Aborting as a precaution against firmware corruption.\n"),
       nProductId, nProductId);
    ret = EXIT_UNRECOGNIZED_PRODUCTID;
    goto done;
  }

done:
  return ret;
}

bool_t
avr2utilIsInServiceMode(
  const DeviceHandle* pDevice)
{
  return pDevice->bInServiceMode;
}

bool_t
avr2utilIsAutoEnterServiceMode(
  const DeviceHandle* pDevice)
{
  return pDevice->bAutoEnterServiceMode;
}

bool_t
avr2utilAllowEnterServiceMode(
  const DeviceHandle* pDevice)
{
  return pDevice->bEnterServiceModeRequestAllowed;
}

bool_t
avr2utilAllowExitServiceMode(
  const DeviceHandle* pDevice)
{
  return pDevice->bExitServiceModeRequestAllowed;
}

void
avr2utilDisplayBuildInfo(
  void)
{
#if defined(AVR2_VIA_IPMI_SUPPORTED)
  const TCHAR** ppszAvailableIPMIBackends;
  API_AVR2IPMI apiAVR2IPMI;
  int ret;
#endif

#if defined(AVR2_VIA_USB_SUPPORTED)
  _tprintf(_T("AVR2 API library: "));
# if defined(AVR2_LIBRARY_STATIC)
  _tprintf(_T("statically linked"));
# else
  _tprintf(_T("dynamically loaded"));
# endif
  _tprintf(_T(", <avr2.h> %d.%d.%d"), AVR2_H_VERSION_SUPER, AVR2_H_VERSION_MAJOR, AVR2_H_VERSION_MINOR);
  _tprintf(_T("\n"));
#endif

#if defined(AVR2_VIA_SMBUS_SUPPORTED)
  _tprintf(_T("AVR2S API library: "));
# if defined(AVR2S_LIBRARY_STATIC)
  _tprintf(_T("statically linked"));
# else
  _tprintf(_T("dynamically loaded"));
# endif
  _tprintf(_T(", <avr2s.h> %d.%d.%d"), AVR2S_H_VERSION_SUPER, AVR2S_H_VERSION_MAJOR, AVR2S_H_VERSION_MINOR);
  _tprintf(_T("\n"));
#endif

#if defined(AVR2_VIA_IPMI_SUPPORTED)
  _tprintf(_T("AVR2IPMI API library: "));
# if defined(AVR2IPMI_LIBRARY_STATIC)
  _tprintf(_T("statically linked"));
# else
  _tprintf(_T("dynamically loaded"));
# endif
  _tprintf(_T(", <avr2ipmi.h> %d.%d.%d"), AVR2IPMI_H_VERSION_SUPER, AVR2IPMI_H_VERSION_MAJOR, AVR2IPMI_H_VERSION_MINOR);
  _tprintf(_T("\n"));

  ret = LoadAVR2IPMILibrary(&apiAVR2IPMI);
  if (EXIT_OK == ret) {
    _tprintf(_T("Available IPMI backends: "));
    ppszAvailableIPMIBackends = apiAVR2IPMI.pfnAVR2IPMI_GetBackends();
    while (NULL != *ppszAvailableIPMIBackends) {
      _tprintf(_T("%s "), *ppszAvailableIPMIBackends);
      ppszAvailableIPMIBackends++;
    }
    _tprintf(_T("\n"));
  } else {
    _tprintf(_T("*** Failed to get list of available IPMI backends\n"));
  }
#endif

  _tprintf(_T("ADB3 API library: "));
#if defined(ADB3_LIBRARY_STATIC)
  _tprintf(_T("statically linked"));
#else
  _tprintf(_T("dynamically loaded"));
#endif
  _tprintf(_T(", <adb3.h> %d.%d.%d"), ADB3_H_VERSION_SUPER, ADB3_H_VERSION_MAJOR, ADB3_H_VERSION_MINOR);
  _tprintf(_T("\n"));

#if defined(AVR2_VIA_ADXDMA_SUPPORTED)
  _tprintf(_T("ADXMDA API library: "));
#if defined(ADXDMA_LIBRARY_STATIC)
  _tprintf(_T("statically linked"));
#else
  _tprintf(_T("dynamically loaded"));
#endif
  _tprintf(_T(", <adxdma.h> %d.%d.%d"), ADXDMA_H_VERSION_SUPER, ADXDMA_H_VERSION_MAJOR, ADXDMA_H_VERSION_MINOR);
  _tprintf(_T("\n"));
#endif
}

