#ifndef _ADATA_AVR2UTIL_CONNECTION_H
#define _ADATA_AVR2UTIL_CONNECTION_H


#include <adb3.h>
#include <adb3/avr2.h>

#include <sdk_common.h>


#if defined(AVR2_VIA_USB_SUPPORTED)
# include <avr2.h>
#endif

#if defined(AVR2_VIA_SMBUS_SUPPORTED)
# include <avr2s.h>
#endif

#if defined(AVR2_VIA_IPMI_SUPPORTED)
# include <avr2ipmi.h>
#endif

#if defined(AVR2_VIA_ADXDMA_SUPPORTED)
# include <adxdma.h>
# include <adxdma/bc.h>
#endif

/* Default I2C chip address where we expect to find the uC */
#define UC_SMBUS_CHIP_ADDR (0x48)

/* Default timeout for a single microcontroller command (microseconds) */
#define DEFAULT_COMMAND_TIMEOUT_US (1000000)

/* Flags for AVR2UTIL_FLAGS environment variable */
#define AVR2UTIL_FLAG_EN_ENTER_SM  (1 << 0)
#define AVR2UTIL_FLAG_EN_EXIT_SM   (1 << 1)
#define AVR2UTIL_FLAG_EN_AUTO_SM   (1 << 2)
#define AVR2UTIL_FLAG_DIS_ENTER_SM (1 << 4)
#define AVR2UTIL_FLAG_DIS_EXIT_SM  (1 << 5)
#define AVR2UTIL_FLAG_DIS_AUTO_SM  (1 << 6)

typedef enum _Avr2utilInterfaceType {
  Avr2utilInterfaceADB3     = 1, /* Via ADB3_* API (PCIe) */
  Avr2utilInterfaceAVR2     = 2, /* Via AVR2_* API (USB or PS UART) */
  Avr2utilInterfaceAVR2S    = 3, /* Via AVR2S_* API (SMBUS) */
  Avr2utilInterfaceAVR2IPMI = 4, /* Via AVR2IPMI_* API (IPMI and then SMBUS from BMC) */
  Avr2utilInterfaceADXDMA   = 5  /* Via ADXDMA_* API (PCIe) */
} Avr2utilInterfaceType;

/* Opaque datatype */
struct _DeviceHandle;
typedef struct _DeviceHandle DeviceHandle;

extern void
avr2utilCheckEnvironment(
  DeviceHandle* pDevice);

extern void
avr2utilCloseDevice(
  DeviceHandle* pDevice);

extern int
avr2utilOpenDeviceADB3(
  unsigned int index,
  bool_t bIndexSpecified,
  unsigned int serial,
  bool_t bSerialSpecified,
  bool_t bVerbose,
  DeviceHandle** ppDevice);

#if !(defined(__VXWORKS__) || defined(__vxworks))

extern int
avr2utilOpenDeviceADXDMA(
  unsigned int index,
  bool_t bVerbose,
  DeviceHandle** ppDevice);

extern int
avr2utilOpenDevicePsuart(
  const TCHAR* pConnectionString,
  bool_t bVerbose,
  DeviceHandle** ppDevice);

#endif

extern int
avr2utilOpenDeviceUSBCom(
  const TCHAR* pDeviceName,
  bool_t bVerbose,
  DeviceHandle** ppDevice);

#if !(defined(__VXWORKS__) || defined(__vxworks))

extern int
avr2utilOpenDeviceSMBUS(
  const TCHAR* pDeviceName,
  uint8_t chipAddr,
  bool_t bVerbose,
  DeviceHandle** ppDevice);

extern int
avr2utilOpenDeviceIPMI(
  const TCHAR* pDeviceName,
  bool_t bVerbose,
  DeviceHandle** ppDevice);

#endif

extern Avr2utilInterfaceType
avr2utilGetInterfaceType(
  const DeviceHandle* pDevice);

extern const TCHAR*
avr2utilGetInterfaceName(
  const DeviceHandle* pDevice);

extern void
avr2utilSetCommandTimeoutUs(
  DeviceHandle* pDevice,
  unsigned int timeoutMs);

extern void
avr2utilRebootDelay(
  DeviceHandle* pDevice);

extern int
avr2utilDetermineServiceMode(
  DeviceHandle* pDevice,
  bool_t bVerbose);

extern bool_t
avr2utilIsInServiceMode(
  const DeviceHandle* pDevice);

extern bool_t
avr2utilIsAutoEnterServiceMode(
  const DeviceHandle* pDevice);

extern bool_t
avr2utilAllowEnterServiceMode(
  const DeviceHandle* pDevice);

extern bool_t
avr2utilAllowExitServiceMode(
  const DeviceHandle* pDevice);

/* Function that performs a transaction with the microcontroller via AVR2_* API (USB) */
extern int
avr2utilSendAvr2Command(
  const DeviceHandle* pDevice,
  bool_t bVerbose,
  uint32_t timeoutUs,
  const uint8_t* pCommand,
  uint32_t commandLength,
  const uint8_t expResponse[2],
  uint8_t* pResponse,
  uint32_t responseLength,
  uint32_t* pActualLength);

/* Function that performs a transaction with the microcontroller via AVR2S_* API (SMBUS) */
extern int
avr2utilSendAvr2CommandEx(
  const DeviceHandle* pDevice,
  bool_t bVerbose,
  uint32_t timeoutUs,
  const uint8_t* pCommand,
  uint32_t commandLength,
   /* If nonzero, changes SMBUS chip address between command and response */
   /* Ignored if not communicating with uC via SMBUS or IPMI+SMBUS */
  uint8_t chipAddrOverride,
  const uint8_t expResponse[2],
  uint8_t* pResponse,
  uint32_t responseLength, /* 0 => Skip receiving response (SMBUS / IPMI+SMBUS only) */
  uint32_t* pActualLength);

#define DRAWING_NUM_BOOTMAN2           (1320) /* Drawing number for AVR2 boot manager */
#define DRAWING_NUM_BOOTMAN2_ALT1       (720) /* Alternative drawing number for AVR2 boot manager */
#define DRAWING_NUM_BOOTMAN2_ALT2   (0x2D000) /* Alternative drawing number for AVR2 boot manager */
#define DRAWING_NUM_BOARDMAN2          (1321) /* Drawing number for AVR2 board manager */
#define DRAWING_NUM_BOARDMAN2_ALT1      (721) /* Alternative drawing number for AVR2 board manager */
#define DRAWING_NUM_BOARDMAN2_ALT2  (0x2D100) /* Alternative drawing number for AVR2 board manager */

extern void
avr2utilDisplayBuildInfo(
  void);

#endif