#ifndef _ADATA_AVR2UTIL_EXITCODE_H
#define _ADATA_AVR2UTIL_EXITCODE_H

#define EXIT_OK                      (0) /* Success */
#define EXIT_UNRECOGNIZED_COMMAND    (1) /* Invalid command passed as 1st positional argument */
#define EXIT_INSUFFICIENT_ARGS       (2) /* Not enough positional arguments following command */
#define EXIT_INVALID_INDEX           (3) /* Index value out of range or not a valid number */
#define EXIT_INVALID_FREQUENCY       (4) /* Frequency value out of range or not a valid number */
#define EXIT_READ_FIRMWARE_FAILED    (5) /* Failed to read firmware from file */
#define EXIT_FIRMWARE_TOO_LARGE      (6) /* Firmware file is too large for uC */
#define EXIT_ALLOCATION_FAILED       (7) /* Failed to allocate buffer for firmware or board config data */
#define EXIT_VERIFY_FAILED           (8) /* Errors found when verifying updated firmware or board config data */
#define EXIT_UNSUPPORTED_MODEL       (9) /* Attempting to use this utility on an unsupported model */
#define EXIT_BAD_COMMAND_LINE       (10) /* Illegal basic command-line syntax */
#define EXIT_USB_NOT_SUPPORTED      (11) /* Access to AVR2 uC via USB is currently not supported for this OS */
#define EXIT_MODE_CHANGE_FAILED     (12) /* AVR2 uC did not enter or exit Service Mode as requested */
#define EXIT_WRONG_MODE             (13) /* Device is in the wrong mode (Service Mode vs. non-Service Mode) for the requested command */
#define EXIT_UNRECOGNIZED_PRODUCTID (14) /* Product ID not recognized; aborting as a precaution against firmware corruption. */
#define EXIT_INVALID_I2C_BUS        (15) /* I2C bus number is not valid */
#define EXIT_INVALID_I2C_DEVICE     (16) /* I2C device number is not valid */
#define EXIT_INVALID_I2C_ADDRESS    (17) /* I2C address is not valid */
#define EXIT_I2C_READ_FILE_FAILED   (18) /* Failed to read data for I2C write from file */
#define EXIT_I2C_WRITE_FILE_FAILED  (19) /* Failed to write data from I2C read to file */
#define EXIT_INVALID_I2C_COUNT      (20) /* I2C data byte count is not valid */
#define EXIT_INVALID_I2C_BYTEVAL    (21) /* I2C data byte value is not valid */
#define EXIT_WRITE_FIRMWARE_FAILED  (22) /* Failed to write firmware/VPD/board config to a file */
#define EXIT_INVALID_OVERRIDE       (23) /* Sensor override value out of range or not a valid number */
#define EXIT_BAD_SPI_SFDP           (24) /* SFDP data from SPI Flash chip does not conform to JESD216/JESD216B */
#define EXIT_INVALID_SPI_INDEX      (25) /* SPI chip index is not valid */
#define EXIT_INVALID_SPI_READ_COUNT (26) /* SPI raw read byte count is not valid */
#define EXIT_INVALID_SPI_WRITE_COUNT (27) /* SPI raw write byte count is not valid */
#define EXIT_INVALID_SPI_BYTEVAL    (28) /* SPI data byte value is not valid */
#define EXIT_INVALID_SPI_ADDRESS    (29) /* SPI address out of range */
#define EXIT_INVALID_SPI_BYTE_COUNT (30) /* SPI program/read/erase byte count out of range or not valid */
#define EXIT_READ_INTEL_HEX_FAILED  (31) /* Failed to read file in Intel Hex (.mcs) format */
#define EXIT_INVALID_TRANSLATION    (32) /* Address translation value is not valid */
#define EXIT_NOT_POWERED            (33) /* Device is not powered on */
#define EXIT_SMBUS_NOT_SUPPORTED    (34) /* SMBUS communication with the AVR2 uC is not supported */
#define EXIT_IPMI_NOT_SUPPORTED     (35) /* IPMI communication with the AVR2 uC is not supported */
#define EXIT_NO_SPI_FLASH           (36) /* No SPI Flash that is accessible from the AVR2 uC */
#define EXIT_INVALID_RAW_READ_COUNT (37) /* "raw" command read byte count not valid */
#define EXIT_INVALID_RAW_BYTEVAL    (38) /* "raw" command bad write byte value */
#define EXIT_INVALID_RECORD_TYPE    (39) /* "display-black-box" bad record type following command */
#define EXIT_INVALID_RECORD_COUNT   (40) /* "display-black-box" bad record count following record type */
#define EXIT_INVALID_BOOTMODE       (41) /* Invalid boot mode specified */
#define EXIT_NOT_CONFIRMED          (42) /* User did not confirm potentially dangerous operation when prompted */
#define EXIT_DEVICE_OPEN_ERROR     (100) /* Failed to open device */
#define EXIT_AVR2_COMMAND_ERROR    (102) /* Failed to send command to AVR2 uC */
#define EXIT_AVR2_BAD_STATUS       (103) /* AVR2 uC returned nonzero status for operation */
#define EXIT_AVR2_SHORT_RESPONSE   (104) /* AVR2 uC's response was too short (< 2 bytes) to be valid */
#define EXIT_LIBRARY_NOT_FOUND     (105) /* Could not find AVR2 or ADB3 shared library/DLL */
#define EXIT_TOO_MANY_COMM_OPTIONS (106) /* Specified more than one communication option, e.g. -smbus and -ipmi */
#define EXIT_BAD_CHIP_ADDR         (107) /* Invalid value passed for -chipaddr option */
#define EXIT_BAD_IPMI_OPTIONS      (108) /* Bad options for opening an IPMI session */
#define EXIT_BAD_TIMEOUT_VALUE     (109) /* Invalid value passed for -timeout option */
#define EXIT_INVALID_BAUD_RATE     (110) /* Invalid value passed for baud rate in -com option connection string */
#define EXIT_SET_BAUD_FAILED       (111) /* Failed to set UART baud rate */
#define EXIT_INVALID_MEZ_INDEX     (112) /* Invalid value passed for mezzanine index */
#define EXIT_INVALID_MEZ_VOLTAGE   (113) /* Invalid value passed for requested / min / max mezzanine I/O rail voltage */
#define EXIT_READ_REG_MAP_FAILED   (114) /* Failed to read clock generator register map file */
#define EXIT_PROBE_REG_MAP_FAILED  (115) /* Failed to determine type of register map file */
#define EXIT_BAD_REG_MAP           (116) /* Register map file contents appear to be invalid */
#define EXIT_BAD_REG_MAP_TYPE      (117) /* Register map file type not recognized */

#endif
