#if defined(_WIN32)

/* Windows */
# include <windows.h>
# include <tchar.h>

#else

/* Linux or VxWorks */
# define _T(x) x
# define _tprintf printf

#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sdk_common.h>

#include "exit_code.h"
#include "fw_version.h"

int
avr2utilVersionCommand(
  const DeviceHandle* pDevice,
  bool_t bVerbose,
  bool_t bNoDisplay,
  uint16_t* pVersion)
{
  uint8_t command[2];
  uint8_t response[10];
  uint8_t expResponse[2];
  uint32_t responseLength;
  uint16_t version[4];
  int ret = EXIT_OK;

  command[0] = 0x03; /* User command group */
  command[1] = 0x83; /* Version query */
  expResponse[0] = 0x03; /* User command group */
  expResponse[1] = 0x03; /* Version response */

  memset(response, 0, sizeof(response));
  ret = avr2utilSendAvr2Command(
    pDevice,
    bVerbose,
    0,
    command,
    sizeof(command),
    expResponse,
    response,
    sizeof(response),
    &responseLength);
  if (EXIT_OK != ret) {
    return ret;
  }

  version[0] = (uint16_t)(((uint16_t)response[2] << 0) | ((uint16_t)response[3] << 8));
  version[1] = (uint16_t)(((uint16_t)response[4] << 0) | ((uint16_t)response[5] << 8));
  version[2] = (uint16_t)(((uint16_t)response[6] << 0) | ((uint16_t)response[7] << 8));
  version[3] = (uint16_t)(((uint16_t)response[8] << 0) | ((uint16_t)response[9] << 8));
  if (!bNoDisplay) {
    _tprintf(_T("AVR2 %s version is %u.%u.%u.%u\n"),
      avr2utilIsInServiceMode(pDevice) ? _T("BootMan2") : _T("BoardMan2"),
      (unsigned int)version[0], (unsigned int)version[1], (unsigned int)version[2], (unsigned int)version[3]);
  }

  if (NULL != pVersion) {
    pVersion[0] = version[0];
    pVersion[1] = version[1];
    pVersion[2] = version[2];
    pVersion[3] = version[3];
  }

  return EXIT_OK;
}
