#ifndef _ADATA_LMK61E2_REGMAP_H
#define _ADATA_LMK61E2_REGMAP_H

#include <sdk_common.h>

#define LMK61E2_MAX_PAIR_COUNT (255)

typedef struct _LMK61E2RegMap {
  uint8_t nPairCount; /* Number of elements in 'pairs' array */
  struct {
    uint8_t nRegIndex;
    uint8_t nRegValue;
  } pairs[1]; /* Variable length array */
} LMK61E2RegMap;

typedef enum _LMK61E2RegMapError {
  LMK61E2RegMapOk               = 0,
  LMK61E2RegMapBadFile          = 1,     /* Did not find any lines in expected format */
  LMK61E2RegMapAllocationError  = 0x100, /* Could not allocate memory for data structures */
  LMK61E2RegMapPermissionDenied = 0x101, /* Could not open file due to permissions of file */
  LMK61E2RegMapNotFound         = 0x102, /* File not found or at least one directory in path not found */
  LMK61E2RegMapOpenFailed       = 0x103, /* Failed to open file due to unexpected error */
  LMK61E2RegMapSeekError        = 0x104, /* Unspecified error occurred when seeking to beginning of file */
  LMK61E2RegMapReadError        = 0x105  /* Unspecified error occurred when reading file */
} LMK61E2RegMapError;

typedef enum _LMK61E2RegMapMessageType {
  LMK61E2RegMapMessageInfo = 0,
  LMK61E2RegMapMessageWarning = 1,
  LMK61E2RegMapMessageError = 2
} LMK61E2RegMapMessageType;

typedef void LMK61E2RegMapMessageCallback(const TCHAR* pMessage, LMK61E2RegMapMessageType type);

LMK61E2RegMapError LMK61E2RegMapReadFromFile(
  FILE* pFile,
  LMK61E2RegMapMessageCallback* pMessageCallback,
  LMK61E2RegMap** ppRegMap);

LMK61E2RegMapError LMK61E2RegMapRead(
  const TCHAR* pFileName,
  LMK61E2RegMapMessageCallback* pMessageCallback,
  LMK61E2RegMap** ppRegMap);

void LMK61E2RegMapDestroy(
  LMK61E2RegMap* pRegMap);

const TCHAR* LM61E2RegMapGetErrorDesc(
  LMK61E2RegMapError nError);

#endif
