#if defined(_WIN32)

/* Windows */
# include <windows.h>
# include <tchar.h>

#else

/* Linux or VxWorks */
# define _T(x) x
# define _tprintf printf

#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sdk_common.h>
#include <platform_endian.h>

#include "exit_code.h"
#include "misc.h"

int
avr2utilTestTimeoutCommand(
  const DeviceHandle* pDevice,
  bool_t bVerbose)
{
  uint8_t command[2];
  uint8_t response[2];
  uint8_t expResponse[2];
  uint32_t responseLength;
  int ret = EXIT_OK;
    
  command[0] = 0xff; /* Invalid command group */
  command[1] = 0x81;
  expResponse[0] = 0xff; /* Invalid command group */
  expResponse[1] = 0x01;

  memset(response, 0, sizeof(response));
  ret = avr2utilSendAvr2Command(
    pDevice,
    bVerbose,
    0,
    command,
    sizeof(command),
    expResponse,
    response,
    sizeof(response),
    &responseLength);
  if (EXIT_AVR2_COMMAND_ERROR == ret) {
    /* We expect this command to fail. */
    ret = EXIT_OK;
    goto done;
  } else if (EXIT_OK == ret) {
    _tprintf(_T("*** Microcontroller command succeeded unexpectedly.\n"));
  } else {
    _tprintf(_T("*** Microcontroller command failed, but not in the expected way.\n"));
  }

  ret = EXIT_AVR2_COMMAND_ERROR;

done:
  return ret;
}

int
avr2utilRawCommand(
  const DeviceHandle* pDevice,
  bool_t bVerbose,
  unsigned int readCount,
  unsigned int writeCount,
  const uint8_t* pWriteBytes)
{
  uint8_t expResponse[2];
  uint32_t responseLength;
  uint8_t* pResponseBuffer = NULL;
  unsigned int i;
  int ret = EXIT_OK;
  
  pResponseBuffer = (uint8_t*)malloc(readCount);
  if (NULL == pResponseBuffer) {
    _tprintf(_T("*** Failed to allocate response buffer.\n"));
    ret = EXIT_ALLOCATION_FAILED;
    goto done;
  }

  expResponse[0] = pWriteBytes[0];
  expResponse[1] = (uint8_t)(pWriteBytes[1] ^ 0x80);

  memset(pResponseBuffer, 0, readCount);

  ret = avr2utilSendAvr2Command(
    pDevice,
    bVerbose,
    0,
    pWriteBytes,
    (uint32_t)writeCount,
    expResponse,
    pResponseBuffer,
    readCount,
    &responseLength);
  if (EXIT_OK != ret) {
    goto done;
  }

  _tprintf(_T("Expected/actual response length = %u / %u\n"), readCount, responseLength);
  for (i = 0; i < readCount; i++) {
    _tprintf(_T("%02X "), pResponseBuffer[i]);
  }
  _tprintf(_T("\n"));

done:
  return ret;
}
