#if defined(_WIN32)

/* Windows */
# include <windows.h>
# include <tchar.h>

#else

/* Linux or VxWorks */
# define _T(x) x
# define _tprintf printf

#endif

#include <stdio.h>
#include <string.h>

#include <sdk_common.h>

#include "exit_code.h"
#include "product_id.h"

int
avr2utilProductIdCommand(
  const DeviceHandle* pDevice,
  bool_t bVerbose,
  bool_t bNoDisplay,
  uint32_t* pProductId)
{
  uint8_t command[2];
  uint8_t response[6];
  uint8_t expResponse[2];
  uint32_t responseLength;
  uint32_t productId;
  int ret = 0;

  command[0] = 0x03; /* User command group */
  command[1] = 0x86; /* Product ID query */
  expResponse[0] = 0x03; /* User command group */
  expResponse[1] = 0x06; /* Product ID response */

  memset(response, 0, sizeof(response));
  ret = avr2utilSendAvr2Command(
    pDevice,
    bVerbose,
    0,
    command,
    sizeof(command),
    expResponse,
    response,
    sizeof(response),
    &responseLength);
  if (ret) {
    return ret;
  }

  productId = (uint32_t)(((uint32_t)response[2] <<  0) | ((uint32_t)response[3] <<  8) |
                         ((uint32_t)response[4] << 16) | ((uint32_t)response[5] << 24));
  if (!bNoDisplay) {
    _tprintf(_T("AVR2 firmware Product ID is %lu(0x%lX)\n"), (unsigned long)productId, (unsigned long)productId);
  }

  if (NULL != pProductId) {
    *pProductId = productId;
  }

  return 0;
}
